#ifndef _LINUX_INTERCEPT_H_
#define _LINUX_INTERCEPT_H_

/****************************************************************************
 *
 * linux_intercept.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (c) 1999-2001 NAI Labs, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of NAI Labs, Inc. nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * This file contains macros which encapsulate the details of "intercepting"
 * system calls.  We "intercept" a system call by replacing the pointer to
 * the kernel's system call routine in the `sys_call_table' with a pointer
 * to a new function of our own devising.  This causes the kernel's system
 * call dispatcher to call our new function rather than the original one
 * whenever a process invokes that system call.
 *
 * 
 *
 ****************************************************************************/

#include <linux/unistd.h>      /* for system call name->number mapping */

/* `sys_call_table' is the kernel's system call table - an array of    *
 * function pointers.  A system call's number is the index of its      *
 * function in the array.  See entry.S in arch-specific assembly area. */
extern void *sys_call_table[];

/* This macro defines a variable to store the original function      *
 * address for the syscall named by `syscall_name'.  When we replace *
 * this syscall later on with the REPLACE_SYSCALL macro, we'll use   *
 * this variable to remember the replaced function address so we can *
 * eventually restore it with the RESTORE_SYSCALL macro.  Make sure  *
 * you use this macro in a scope which is visible from the places    *
 * where you use the REPLACE/RESTORE macros.                         */

#define SETUP_SYSCALL_STORAGE( syscall_name ) \
   void *orig_ ## syscall_name

/* Use the REPLACE_SYSCALL macro to make the syscall named by  *
 * `syscall_name' correspond to the function `replacement_fxn' */

#define REPLACE_SYSCALL( syscall_name, replacement_fxn ) \
   orig_ ## syscall_name = sys_call_table[ __NR_ ## syscall_name ]; \
   sys_call_table[ __NR_ ## syscall_name ] = (void *)replacement_fxn

/* Use the RESTORE_SYSCALL macro to undo the effects of REPLACE_SYSCALL */

#define RESTORE_SYSCALL( syscall_name ) \
   sys_call_table[ __NR_ ## syscall_name ] = orig_ ## syscall_name

#endif



