#########################################################################
#
# LOMAC - Low Water-Mark Mandatory Access Control for Linux 
# Copyright (c) 1999-2001 NAI Labs, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#   Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# 
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# 
#   Neither the name of NAI Labs, Inc. nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# This module contains a number of common predicates that are useful
# to LOMAC test scripts.
#
# To use, put this in your Perl test script:
# use lomacpred;
#
#########################################################################

package lomacpred;   # I am package "lomacpred"
use Exporter ();     # I depend Exporter, but don't import any Exporter symbols
@ISA = qw(Exporter); # I'm an Exporter, and inherit Exporter's import() method
@EXPORT = (ProcAtLevelPred, FileAtLevelPred);  # I export these symbols


#########################################################################
#
# Subroutine ProcAtLevelPred
#
# Parms:
# $_[ 0 ]   level we expect to be at
#
# Return:   1 if we (that is, this process) is running at the level
#           indicated by $_[ 0 ], otherwise 0.
#
#########################################################################

sub ProcAtLevelPred {
        local($requested_level) = pop(@_);
        local($our_level);

        # Use the level program to determine our level.  $$ means our pid.
        open( LEVELOUT, "/opt/lomac/bin/lps $$|" );
	@lps_output = <LEVELOUT>;
	close( LEVELOUT );
	$level_line = $lps_output[ 1 ];
	@level_cols = split( ' ', $level_line );
	$our_level = $level_cols[ 1 ];

        return( $our_level == $requested_level );
}


#########################################################################
#
# Subroutine FileAtLevelPred
#
# Parms:
# $_[ 0 ]   filename whose level we want to check
# $_[ 1 ]   level we expect the file to be at
#
# Return:   1 if the file indicated by $_[ 0 ] is at the level
#           indicated by $_[ 1 ], otherwise 0.
#
#########################################################################

sub FileAtLevelPred {
	local($file_level) = pop(@_);
	local($filename) = pop(@_);
	local($level);

	# Use the level program to determine the file's level.
	open( LEVELOUT, "/opt/lomac/bin/lls $filename|" );
	$level_line = <LEVELOUT>;
	close( LEVELOUT );
	@level_cols = split( ' ', $level_line );
	$level = $level_cols[ 3 ];

	return( $level == $file_level );
}
